package com.nisum.statisticsutils.programs;

import com.mongodb.client.FindIterable;
import com.mongodb.client.model.Filters;
import com.nisum.statisticsutils.utils.*;
import org.bson.Document;

import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Properties;



public class DailyReport {

	public class Mail{

		private String storeNumbers;
		private String reportSinceDate;
		private String sendTo;
		private String sendFrom;
		private String smtpServer;
		private int smtpPort;

		public String getSendTo() {
			return sendTo;
		}

		public void setSendTo(String sendTo) {
			this.sendTo = sendTo;
		}

		public String getSendFrom() {
			return sendFrom;
		}

		public void setSendFrom(String sendFrom) {
			this.sendFrom = sendFrom;
		}

		public void setReportSinceDate(String sinceDate) {
			this.reportSinceDate = sinceDate;
		}

		public String getSmtpServer() {
			return smtpServer;
		}

		public void setSmtpServer(String smtpServer) {
			this.smtpServer = smtpServer;
		}

		public int getSmtpPort() {
			return smtpPort;
		}

		public void setSmtpPort(int smtpPort) {
			this.smtpPort = smtpPort;
		}

		public String getReportSinceDate() {
			return this.reportSinceDate;
		}
		public String getStoreNumbers() {
			return storeNumbers;
		}

		public void setStoreNumbers(String storeNumbers) {
			this.storeNumbers = storeNumbers;
		}

	}

	public Mail getProperties() throws Exception {
		Properties prop = new Properties();
		String propFile = "config.properties";
		InputStream input = getClass().getClassLoader().getResourceAsStream(propFile);
		if(input==null){
			throw new FileNotFoundException("config.properties file not found");
		}else{
			prop.load(input);
		}

		Mail m = new Mail();
		m.setSendFrom(prop.getProperty("sendFrom"));
		m.setSendTo(prop.getProperty("sendTo"));
		m.setSmtpServer(prop.getProperty("smtpServer"));
		m.setSmtpPort(Integer.parseInt(prop.getProperty("smtpPort")));
		m.setReportSinceDate(prop.getProperty("reportSinceDate"));
		m.setStoreNumbers(prop.getProperty("storeNumbers"));
		return m;
	}

	public static void main(String[] args) throws Exception {
		Mail m = new DailyReport().getProperties();
		String sinceDate=m.getReportSinceDate();
		String[] stores = m.getStoreNumbers().split(",");
		MongoUtils mo = new MongoUtils();
		JsonUtils ju = new JsonUtils();
		//number of records after date
		System.out.println("Fetching number of transactions since date "+ sinceDate +" for each store");
		String emailBody="";
		Long numOfTxns=0l ;
		emailBody = emailBody + ("Report of transactions since date: " + sinceDate);
		emailBody = emailBody + ("\n\nNumber of Taxation Calls for stores");
		for(String store:stores) {
			numOfTxns = mo.getCountOfMatchingRecords(Filters.and(Filters.gte("timestamp",sinceDate),Filters.eq("storeId",store)));
			emailBody = emailBody + ("\n\tStore# " + store + "\t:" +  numOfTxns);
		}

		System.out.println("Fetching errors since date "+ sinceDate +" for all given stores");
		//number of records after date with exceptions in response
		numOfTxns = mo.getCountOfMatchingRecords(Filters.and(Filters.gte("timestamp",sinceDate),Filters.regex("response", "exception")));
		emailBody = emailBody + ("\n\nNumber of Transactions with Exceptions: " + numOfTxns);
		FindIterable<Document> errors = mo.getAllMatchingDocuments(Filters.and(Filters.gte("timestamp",sinceDate),Filters.regex("response", "exception")));
		for(Document d: errors) {
			String error = (ju.searchJsonStringByJsonPath(d.toJson(), "$.response"));
			String[] errorz = error.split(":");
			String traceId = errorz[errorz.length-1].replace("\"", "").replace("}", "");
			Document errorDoc = mo.getMatchingDocument(Filters.eq("_id",traceId));
			emailBody = emailBody + ("\n" + ju.searchJsonStringByJsonPath(errorDoc.toJson(),"$.timestamp") + "::" + error );
		}
		
		//PRINTOUT THE ERRORS

		System.out.println("Fetching error counts since date "+ sinceDate +" for all given stores");
		//number of records after date with errors in response and percentage > 25.
		numOfTxns = mo.getCountOfMatchingRecords(Filters.and(Filters.gte("timestamp",sinceDate),Filters.gt("percentage",25)));
		emailBody = emailBody + ("\n\nNumber of Erroneous Transactions with 25% or more missing UPCs: " + numOfTxns);

		System.out.println("Missing UPS "+ sinceDate +" for all given stores");
		//list of unique missing UPCs from the records since date
		FindIterable<Document> allMissingUPCs = mo.getAllMatchingDocuments(Filters.and(Filters.gte("timestamp",sinceDate),Filters.exists("upc_bpn")));
		ArrayList<String> UPCs = new ArrayList<String>();
		emailBody = emailBody + ("\n\nMissing UPCs:\n");
		for(Document d:allMissingUPCs) {
			String upc = ju.searchJsonStringByJsonPath(d.toJson(), "$.upc_bpn");
			String[] arrupc = upc.split(",");
			for(String s:arrupc) {
				if(!UPCs.contains(s)) {
					UPCs.add(s);
					emailBody = emailBody + (s + "," ) ;
				}
			}
		}
		emailBody = emailBody + ("\n\nNumber of UPCs missing product classes: " + UPCs.size());
		emailBody = emailBody + ("\n\nTaxation Report generated at : " + new java.util.Date());
		mo.closeConnection();
		System.out.println("Generating Emails");
		EmailUtils.sendMail(m.getSmtpServer(),m.getSmtpPort(),m.sendFrom,m.sendTo,"Taxation Report: " + new java.util.Date(),emailBody,"");
		System.out.println("Process Complete " + new java.util.Date());
	}
	
}
