package com.nisum.statisticsutils.programs;

import com.mongodb.client.FindIterable;
import com.mongodb.client.model.Filters;
import com.nisum.statisticsutils.models.ExcelCells;
import com.nisum.statisticsutils.utils.EmailUtils;
import com.nisum.statisticsutils.utils.ExcelWriteUtils;
import com.nisum.statisticsutils.utils.JsonUtils;
import com.nisum.statisticsutils.utils.MongoUtils;
import j2html.tags.ContainerTag;
import org.apache.commons.mail.EmailException;
import org.bson.Document;

import javax.mail.MessagingException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static j2html.TagCreator.body;
import static j2html.TagCreator.h2;
import static j2html.TagCreator.head;
import static j2html.TagCreator.html;
import static j2html.TagCreator.table;
import static j2html.TagCreator.td;
import static j2html.TagCreator.th;
import static j2html.TagCreator.tr;


public class DailyReportInExcelAndEmailv2 {

	public class Mail{

		private String storeNumbers;
		private String reportSinceDate;
		private String sendTo;
		private String sendFrom;
		private String smtpServer;
		private int smtpPort;

		public String getSendTo() {
			return sendTo;
		}

		public void setSendTo(String sendTo) {
			this.sendTo = sendTo;
		}

		public String getSendFrom() {
			return sendFrom;
		}

		public void setSendFrom(String sendFrom) {
			this.sendFrom = sendFrom;
		}

		public void setReportSinceDate(String sinceDate) {
			this.reportSinceDate = sinceDate;
		}

		public String getSmtpServer() {
			return smtpServer;
		}

		public void setSmtpServer(String smtpServer) {
			this.smtpServer = smtpServer;
		}

		public int getSmtpPort() {
			return smtpPort;
		}

		public void setSmtpPort(int smtpPort) {
			this.smtpPort = smtpPort;
		}

		public String getReportSinceDate() {
			return this.reportSinceDate;
		}
		public String getStoreNumbers() {
			return storeNumbers;
		}

		public void setStoreNumbers(String storeNumbers) {
			this.storeNumbers = storeNumbers;
		}

	}

	ArrayList<String> txn25pc;
	ArrayList<String> errors;
	ArrayList<String> missingUPCs;
	ExcelWriteUtils ewu;
	Mail m;
	MongoUtils mo;
	JsonUtils ju;
	String[] stores;
	String emailBody;
	ContainerTag emailHtml;
	String sinceDate;
	HashMap<String,Long> hmTxnCount;
	HashMap<String,Long> hmTxnCount25pc;
	HashMap<String,Long> hmErrorCount;

	public DailyReportInExcelAndEmailv2() throws Exception {
		txn25pc = new ArrayList<String>();
		errors = new ArrayList<String>();
		missingUPCs = new ArrayList<String>();
		ewu = new ExcelWriteUtils();
		m = getProperties();
		mo = new MongoUtils();
		ju = new JsonUtils();

		stores = m.getStoreNumbers().split(",");
		emailBody = "";
		emailHtml = null;
		sinceDate = m.getReportSinceDate();
		hmTxnCount = new HashMap<String,Long>();
		hmTxnCount25pc = new HashMap<String,Long>();
		hmErrorCount = new HashMap<String,Long>();

	}

	public Mail getProperties() throws Exception {
		Properties prop = new Properties();
		String propFile = "config.properties";
		ClassLoader loader = Thread.currentThread().getContextClassLoader();
		InputStream input = getClass().getClassLoader().getResourceAsStream(propFile);
		if(input==null){
			throw new FileNotFoundException("config.properties file not found");
		}else{
			prop.load(input);
		}

		Mail m = new Mail();
		m.setSendFrom(prop.getProperty("sendFrom"));
		m.setSendTo(prop.getProperty("sendTo"));
		m.setSmtpServer(prop.getProperty("smtpServer"));
		m.setSmtpPort(Integer.parseInt(prop.getProperty("smtpPort")));
		m.setReportSinceDate(prop.getProperty("reportSinceDate"));
		m.setStoreNumbers(prop.getProperty("storeNumbers"));
		return m;
	}

	public String[] splitUPCs(String upc){
		return upc.split(",");
	}

	public ArrayList<String> fetchErrors() throws InterruptedException {
		for (String store : stores) {
			long i = 0l;
			FindIterable<Document> docs = mo.getAllMatchingDocuments(
				Filters.or(
					Filters.and(
							Filters.gte("timestamp", sinceDate),
							Filters.eq("storeId", store),
							Filters.regex("response", "errors"),
							Filters.regex("response", "lineItems")
					),
					Filters.and(
							Filters.gte("timestamp", sinceDate),
							Filters.eq("storeId", store),
							Filters.regex("response", "exception"),
							Filters.regex("response", "SocketException")
					)
				)
			);

			for (Document d : docs) {
				errors.add(d.toJson());
				i++;
			}
			hmErrorCount.put(store,i);
		}
		return errors;
	}

	public ArrayList<String> fetchTxnsHigherThan25PC() throws Exception{

		for (String store : stores) {
			Long i = 0l;
			FindIterable<Document> docs = mo.getAllMatchingDocuments(
					Filters.and(
							Filters.gte("timestamp", sinceDate),
							Filters.eq("storeId",store),
							Filters.gt("percentage", 25)
					)
			);

			for(Document d:docs){
				txn25pc.add(d.toJson());
				i++;
			}
			hmTxnCount25pc.put(store,i);
		}

		return txn25pc;
	}

	public void saveAndCloseWorkBook(String reportPath) throws IOException {
		ewu.saveWorkBook(reportPath);
		ewu.closeWorkBook();
	}

	public static String replaceGroup(String regex, String source, int groupToReplace, String replacement) throws Exception{
			return replaceGroup(regex, source, groupToReplace, 1, replacement);
	}

	public static String replaceGroup(String regex, String source, int groupToReplace, int groupOccurrence, String replacement) throws Exception{
		Matcher m = Pattern.compile(regex).matcher(source);
			for (int i = 0; i < groupOccurrence; i++)
				if (!m.find()) return source; // pattern not met, may also throw an exception here
			return new StringBuilder(source).replace(m.start(groupToReplace), m.end(groupToReplace), replacement).toString();
	}

	public void populateErrorInExcel() throws Exception {
		ewu.createSheet("Errors");
		HashMap<Integer, String> hmCols = new HashMap<Integer,String>();
		hmCols.put(0,"Store#");
		hmCols.put(1,"Error Date");
		hmCols.put(2,"Error Details");

		ExcelCells ec = new ExcelCells(-1,hmCols);
		ArrayList<ExcelCells> aec = new ArrayList<ExcelCells>();
		aec.add(ec);

		for(String s:this.errors){
			try{
				String storeId = ju.searchJsonStringByJsonPath(s,"$.storeId");
				String ts = ju.searchJsonStringByJsonPath(s,"$.timestamp");
				String er = ju.searchJsonStringByJsonPath(s,"$.response");

				hmCols = new HashMap<Integer,String>();
				hmCols.put(0,storeId);
				hmCols.put(1,ts);
				hmCols.put(2,er);


				ec = new ExcelCells(-1,hmCols);
				aec.add(ec);
			}catch(Exception e){
				System.out.println("Error fetching error details for the following document: \n" + s);
				System.out.println("==================================================================================");
			}
		}
		ewu.populateCells(aec);
	}

	public String padLeftZeros(String inputString, int length) {
		if (inputString.length() >= length) {
			return inputString;
		}
		StringBuilder sb = new StringBuilder();
		while (sb.length() < length - inputString.length()) {
			sb.append(' ');
		}
		sb.append(inputString);

		return sb.toString();
	}

	public void populateMissingUPCsInExcel() throws Exception{
		ewu.createSheet("Missing UPCs");
		HashMap<Integer, String> hmCols = new HashMap<Integer,String>();
		hmCols.put(0,"Missing UPC - (from Txns with 25+ percent missing UPCs)");

		ExcelCells ec = new ExcelCells(-1,hmCols);
		ArrayList<ExcelCells> aec = new ArrayList<ExcelCells>();
		aec.add(ec);

		for(String s:this.txn25pc){
			try{
				String missingUPCs = ju.searchJsonStringByJsonPath(s,"$.upc_bpn");
				String[] arrUPC = missingUPCs.split(",");
				for(String upc:arrUPC){
					hmCols = new HashMap<Integer,String>();
					if(!this.missingUPCs.contains(upc)){
						this.missingUPCs.add(upc);
						hmCols.put(0,upc);
						ec = new ExcelCells(-1,hmCols);
						aec.add(ec);
					}


				}
			}catch(Exception e){
				System.out.println("Error fetching txn with 25pc more missing UPCs for the following document: \n" + s);
				System.out.println("==================================================================================");
			}
		}
		ewu.populateCells(aec);
	}

	public HashMap<String,Long> fetchTxnCountsPerStore()throws Exception{
		for(String store:stores){
			Long txnCount = mo.getCountOfMatchingRecords(Filters.and(Filters.gte("timestamp", sinceDate), Filters.eq("storeId", store)));
			hmTxnCount.put(store,txnCount);
		}
		return hmTxnCount;
	}

	public void appendToEmailBody(String s){
		this.emailBody = emailBody +  s ;
	}

	public void sendEmail(String reportPath) throws EmailException {
		appendToEmailBody( "Taxation Report since - " + this.sinceDate);
		appendToEmailBody("\n" +
				padLeftZeros("Store#",6) + "|" +
				padLeftZeros("Total Txn Count",15) + "|" +
				padLeftZeros(">25% PrdCls Mismatch",20) + "|" +
				padLeftZeros("Error Count",11) + "|" +
				padLeftZeros("PrdCls Mismatch %",17) + "|" +
				padLeftZeros("Error %",7));



		Integer totaltxn25pc=0;
		Integer totalErrors=0;
		Integer totalOfTotalTxns=0;
		for(String store: this.stores){
			String totalTxnCnt =this.hmTxnCount.get(store)==null?"0":this.hmTxnCount.get(store).toString();
			totalOfTotalTxns = totalOfTotalTxns + Integer.parseInt(totalTxnCnt);
			String txn25pcCnt = this.hmTxnCount25pc.get(store)==null?"0":this.hmTxnCount25pc.get(store).toString();
			totaltxn25pc = totaltxn25pc + Integer.parseInt(txn25pcCnt);
			String errorCnt = this.hmErrorCount.get(store)==null?"0": this.hmErrorCount.get(store).toString();
			totalErrors = totalErrors + Integer.parseInt(errorCnt);
			String pcOf25pc = String.valueOf(Integer.parseInt(txn25pcCnt)*100/Integer.parseInt(totalTxnCnt)) + "%";
			if(pcOf25pc.trim().equals("0%")){
				pcOf25pc="<1%";
			}
			String pcOfErrCnt = String.valueOf(Integer.parseInt(errorCnt)*100/Integer.parseInt(totalTxnCnt)) + "%";
			if(pcOfErrCnt.trim().equals("0%")){
				pcOfErrCnt="<1%";
			}



			this.emailBody = this.emailBody + "\n"+
					padLeftZeros(store,6) + "|" +
					padLeftZeros(totalTxnCnt,15) + "|" +
					padLeftZeros(txn25pcCnt.toString(),20) + "|" +
					padLeftZeros(errorCnt,11) + "|" +
					padLeftZeros(pcOf25pc,17) + "|" +
					padLeftZeros(pcOfErrCnt,7);
		}



		String overAllPercentError = String.valueOf((totaltxn25pc*100)/totalOfTotalTxns);
		if(overAllPercentError.equals("0")){
			overAllPercentError = "<1";
		}


		this.emailBody = this.emailBody + "\n" +
				padLeftZeros("Total",6) + "|" +
				padLeftZeros(totalOfTotalTxns.toString(),15) + "|" +
				padLeftZeros(totaltxn25pc.toString(),20) + "|" +
				padLeftZeros(totalErrors.toString(),11) + "|" +
				padLeftZeros(String.valueOf((totaltxn25pc*100)/totalOfTotalTxns) + "%",17) + "|" +
				padLeftZeros(overAllPercentError + "%",7);

		EmailUtils.sendMail(m.getSmtpServer(),m.getSmtpPort(),m.sendFrom,m.sendTo,"Taxation Report: " + new Date(),emailBody,reportPath);

	}

	public void sendHtmlEmail(String reportPath) throws EmailException, IOException, MessagingException {
		emailHtml = table()
				.withStyle("border: 1px solid black")
				.withId("results").with(tr
				(
						th("Store#").withStyle("border: 1px solid black;text-align: center;font-weight:bold;width:50px"),
						th("Total Txn Count").withStyle("border: 1px solid black;text-align: center;font-weight:bold;width:100px"),
						th("Txn w:25+% Missing UPCs").withStyle("border: 1px solid black;text-align: center;font-weight:bold;width:100px"),
						th("Error Count").withStyle("border: 1px solid black;text-align: center;font-weight:bold;width:100px"),
						th("% of Txns w:25+% Missing UPCs").withStyle("border: 1px solid black;text-align: center;font-weight:bold;width:100px"),
						th("% of Txns w:Errors").withStyle("border: 1px solid black;text-align: center;font-weight:bold;width:100px")

				));

		Integer totaltxn25pc=0;
		Integer totalErrors=0;
		Integer totalOfTotalTxns=0;
		for(String store: this.stores){
			String totalTxnCnt =this.hmTxnCount.get(store)==null?"0":this.hmTxnCount.get(store).toString();
			totalOfTotalTxns = totalOfTotalTxns + Integer.parseInt(totalTxnCnt);
			String txn25pcCnt = this.hmTxnCount25pc.get(store)==null?"0":this.hmTxnCount25pc.get(store).toString();
			totaltxn25pc = totaltxn25pc + Integer.parseInt(txn25pcCnt);
			String errorCnt = this.hmErrorCount.get(store)==null?"0": this.hmErrorCount.get(store).toString();
			totalErrors = totalErrors + Integer.parseInt(errorCnt);
			String pcOf25pc = String.valueOf(Integer.parseInt(txn25pcCnt)*100/Integer.parseInt(totalTxnCnt));
			if(pcOf25pc.trim().equals("0")){
				pcOf25pc="<1";
			}
			String pcOfErrCnt = String.valueOf(Integer.parseInt(errorCnt)*100/Integer.parseInt(totalTxnCnt));
			if(pcOfErrCnt.trim().equals("0")){
				pcOfErrCnt="<1";
			}
			this.emailHtml.with(tr(td().withStyle("border: 1px solid black;text-align: left").withText(store),td().withStyle("border: 1px solid black;text-align: right").withText(totalTxnCnt),td().withStyle("border: 1px solid black;text-align: right").withText(txn25pcCnt),td().withStyle("border: 1px solid black;text-align: right").withText(errorCnt),td().withStyle("border: 1px solid black;text-align: right").withText(pcOf25pc + "%"),td().withStyle("border: 1px solid black;text-align: right").withText(pcOfErrCnt+"%")));
		}

		String overAllPercentError = String.valueOf((totalErrors*100)/totalOfTotalTxns);
		if(overAllPercentError.equals("0")){
			overAllPercentError = "<1";
		}
		this.emailHtml.with(tr(td("Total").withStyle("border: 1px solid black;text-align: left;font-weight:bold"),td(totalOfTotalTxns.toString()).withStyle("border: 1px solid black;text-align: right;font-weight:bold"),td(totaltxn25pc.toString()).withStyle("border: 1px solid black;text-align: right;font-weight:bold"),td(totalErrors.toString()).withStyle("border: 1px solid black;text-align: right;font-weight:bold"),td(String.valueOf((totaltxn25pc*100)/totalOfTotalTxns) + "%").withStyle("border: 1px solid black;text-align: right;font-weight:bold"),td(overAllPercentError + "%").withStyle("border: 1px solid black;text-align: right;font-weight:bold")));
		ContainerTag finalHtml = html().with(head().with(h2("Taxation Report since - " + this.sinceDate))).with(body().with(this.emailHtml));
		EmailUtils.sendHtmlMail(m.getSmtpServer(),m.getSmtpPort(),m.sendFrom,m.sendTo,"Taxation Report: " + new Date(),finalHtml.render().toString(),reportPath);
	}




	public static void main(String[] args) throws Exception {
		DailyReportInExcelAndEmailv2 dailyReport = new DailyReportInExcelAndEmailv2();
		String reportPath = System.getProperty("user.dir") + "/AllStoreUPCandErrorReport.xlsx";

		System.out.println("Fetching txn counts for each store");
		dailyReport.fetchTxnCountsPerStore();
		System.out.println("Fetching txn counts with 25% or more missing UPCs for each store");
		dailyReport.fetchTxnsHigherThan25PC();
		System.out.println("Fetching errors from txns for each store");
		dailyReport.fetchErrors();

		System.out.println("Populating errors in excel");
		dailyReport.populateErrorInExcel();
		System.out.println("Populating missing UPCs in excel");
		dailyReport.populateMissingUPCsInExcel();

		System.out.println("Saving & closing the workbook");
		dailyReport.saveAndCloseWorkBook(reportPath);
		//dailyReport.sendEmail(reportPath);
		System.out.println("Sening Email");
		dailyReport.sendHtmlEmail(reportPath);





	}
}

